@@ -1,5 +1,5 @@
 #  Pyrogram - Telegram MTProto API Client Library for Python
-#  Copyright (C) 2017-2020 Dan <https://github.com/delivrance>
+#  Copyright (C) 2017-present Dan <https://github.com/delivrance>
 #
 #  This file is part of Pyrogram.
 #
@@ -18,6 +18,7 @@
 
 import asyncio
 import logging
+from typing import Optional
 
 from .transport import *
 from ..session.internals import DataCenter
@@ -36,12 +37,13 @@ class Connection:
         4: TCPIntermediateO
     }
 
-    def __init__(self, dc_id: int, test_mode: bool, ipv6: bool, proxy: dict, mode: int = 3):
+    def __init__(self, dc_id: int, test_mode: bool, ipv6: bool, proxy: dict, media: bool = False, mode: int = 3):
         self.dc_id = dc_id
         self.test_mode = test_mode
         self.ipv6 = ipv6
         self.proxy = proxy
-        self.address = DataCenter(dc_id, test_mode, ipv6)
+        self.media = media
+        self.address = DataCenter(dc_id, test_mode, ipv6, media)
         self.mode = self.MODES.get(mode, TCPAbridged)
 
         self.protocol = None  # type: TCP
@@ -54,30 +56,31 @@ class Connection:
                 log.info("Connecting...")
                 await self.protocol.connect(self.address)
             except OSError as e:
-                log.warning(e)  # TODO: Remove
-                self.protocol.close()
+                log.warning(f"Unable to connect due to network issues: {e}")
+                await self.protocol.close()
                 await asyncio.sleep(1)
             else:
-                log.info("Connected! {} DC{} - IPv{} - {}".format(
+                log.info("Connected! {} DC{}{} - IPv{} - {}".format(
                     "Test" if self.test_mode else "Production",
                     self.dc_id,
+                    " (media)" if self.media else "",
                     "6" if self.ipv6 else "4",
-                    self.mode.__name__
+                    self.mode.__name__,
                 ))
                 break
         else:
             log.warning("Connection failed! Trying again...")
             raise TimeoutError
 
-    def close(self):
-        self.protocol.close()
+    async def close(self):
+        await self.protocol.close()
         log.info("Disconnected")
 
     async def send(self, data: bytes):
         try:
             await self.protocol.send(data)
-        except Exception:
-            raise OSError
+        except Exception as e:
+            raise OSError(e)
 
-    async def recv(self) -> bytes or None:
+    async def recv(self) -> Optional[bytes]:
         return await self.protocol.recv()
